@@ -1,4 +1,4 @@
-/* $Cambridge: exim/exim-src/src/sieve.c,v 1.23 2006/10/10 15:36:50 ph10 Exp $ */
+/* $Cambridge: exim/exim-src/src/sieve.c,v 1.27 2007/04/12 09:00:52 ph10 Exp $ */
 
 /*************************************************
 *     Exim - an Internet mail transport agent    *
@@ -28,11 +28,14 @@
 /* Undefine it for UNIX-style \n end-of-line terminators (default). */
 #undef RFC_EOL
 
+/* Define this for development of the Sieve extension "encoded-character". */
+#undef ENCODED_CHARACTER
+
 /* Define this for development of the Sieve extension "envelope-auth". */
 #undef ENVELOPE_AUTH
 
-/* Define this for development of the Sieve extension "notify".     */
-#undef NOTIFY
+/* Define this for development of the Sieve extension "enotify".    */
+#undef ENOTIFY
 
 /* Define this for the Sieve extension "subaddress".                */
 #define SUBADDRESS
@@ -58,11 +61,14 @@ struct Sieve
   int keep;
   int require_envelope;
   int require_fileinto;
+#ifdef ENCODED_CHARACTER
+  int require_encoded_character;
+#endif
 #ifdef ENVELOPE_AUTH
   int require_envelope_auth;
 #endif
-#ifdef NOTIFY
-  int require_notify;
+#ifdef ENOTIFY
+  int require_enotify;
   struct Notification *notified;
 #endif
 #ifdef SUBADDRESS
@@ -97,11 +103,12 @@ struct String
 struct Notification
   {
   struct String method;
-  struct String priority;
+  struct String importance;
   struct String message;
   struct Notification *next;
   };
 
+static int eq_asciicase(const struct String *needle, const struct String *haystack, int match_prefix);
 static int parse_test(struct Sieve *filter, int *cond, int exec);
 static int parse_commands(struct Sieve *filter, int exec, address_item **generated);
 
@@ -125,13 +132,17 @@ static uschar str_fileinto_c[]="fileinto";
 static const struct String str_fileinto={ str_fileinto_c, 8 };
 static uschar str_envelope_c[]="envelope";
 static const struct String str_envelope={ str_envelope_c, 8 };
+#ifdef ENCODED_CHARACTER
+static uschar str_encoded_character_c[]="encoded-character";
+static const struct String str_encoded_character={ str_encoded_character_c, 17 };
+#endif
 #ifdef ENVELOPE_AUTH
 static uschar str_envelope_auth_c[]="envelope-auth";
 static const struct String str_envelope_auth={ str_envelope_auth_c, 13 };
 #endif
-#ifdef NOTIFY
-static uschar str_notify_c[]="notify";
-static const struct String str_notify={ str_notify_c, 6 };
+#ifdef ENOTIFY
+static uschar str_enotify_c[]="enotify";
+static const struct String str_enotify={ str_enotify_c, 7 };
 #endif
 #ifdef SUBADDRESS
 static uschar str_subaddress_c[]="subaddress";
@@ -256,6 +267,48 @@ for (pass=0; pass<=1; ++pass)
 }
 
 
+/*************************************************
+*     Check mail address for correct syntax      *
+*************************************************/
+
+/*
+Check mail address for being syntactically correct.
+
+Arguments:
+  filter      points to the Sieve filter including its state
+  address     String containing one address
+
+Returns
+  1           Mail address is syntactically OK
+ -1           syntax error
+*/
+
+int check_mail_address(struct Sieve *filter, const struct String *address)
+{
+int start, end, domain;
+uschar *error,*ss;
+
+if (address->length>0)
+  {
+  ss = parse_extract_address(address->character, &error, &start, &end, &domain,
+    FALSE);
+  if (ss == NULL)
+    {
+    filter->errmsg=string_sprintf("malformed address \"%s\" (%s)",
+      address->character, error);
+    return -1;
+    }
+  else
+    return 1;
+  }
+else
+  {
+  filter->errmsg=CUS "empty address";
+  return -1;
+  }
+}
+
+
 /*************************************************
 *          Decode URI encoded string             *
 *************************************************/
@@ -269,6 +322,7 @@ Returns
  -1                 Encoding error
 */
 
+#ifdef ENOTIFY
 static int uri_decode(struct String *str)
 {
 uschar *s,*t,*e;
@@ -312,19 +366,28 @@ Parse mailto-URI.
 Arguments:
   filter      points to the Sieve filter including its state
   uri         URI, excluding scheme
+  recipient
+  body
 
 Returns
   1           URI is syntactically OK
+  0           Unknown URI scheme
  -1           syntax error
 */
 
-int parse_mailto_uri(struct Sieve *filter, const uschar *uri, string_item **recipient, struct String *body)
+static int parse_mailto_uri(struct Sieve *filter, const uschar *uri, string_item **recipient, struct String *header, struct String *body)
 {
 const uschar *start;
 struct String to,hname,hvalue;
 int capacity;
 string_item *new;
 
+if (Ustrncmp(uri,"mailto:",7))
+  {
+  filter->errmsg=US "Unknown URI scheme";
+  return 0;
+  }
+uri+=7;
 if (*uri && *uri!='?')
   for (;;)
     {
@@ -400,7 +463,7 @@ if (*uri=='?')
         return -1;
         }
       }
-    if (hname.length==2 && strcmp(CS hname.character,"to")==0)
+    if (hname.length==2 && strcmpic(hname.character, US"to")==0)
       {
       new=store_get(sizeof(string_item));
       new->text=store_get(hvalue.length+1);
@@ -409,8 +472,31 @@ if (*uri=='?')
       new->next=*recipient;
       *recipient=new;
       }
-    else if (hname.length==4 && strcmp(CS hname.character,"body")==0)
+    else if (hname.length==4 && strcmpic(hname.character, US"body")==0)
       *body=hvalue;
+    else
+      {
+      static struct String ignore[]=
+        {
+        {US"from",4},
+        {US"subject",7},
+        {US"received",8}
+        };
+      static struct String *end=ignore+sizeof(ignore)/sizeof(ignore[0]);
+      struct String *i;
+
+      for (i=ignore; i<end && !eq_asciicase(&hname,i,0); ++i);
+      if (i==end)
+        {
+        if (header->length==-1) header->length=0;
+        capacity=header->length;
+        header->character=string_cat(header->character,&capacity,&header->length,hname.character,hname.length);
+        header->character=string_cat(header->character,&capacity,&header->length,CUS ": ",2);
+        header->character=string_cat(header->character,&capacity,&header->length,hvalue.character,hvalue.length);
+        header->character=string_cat(header->character,&capacity,&header->length,CUS "\n",1);
+        header->character[header->length]='\0';
+        }
+      }
     if (*uri=='&') ++uri;
     else break;
     }
@@ -422,6 +508,8 @@ if (*uri)
   }
 return 1;
 }
+#endif
+
 
 /*************************************************
 *          Octet-wise string comparison          *
@@ -1107,8 +1195,214 @@ return 1;
 }
 
 
+#ifdef ENCODED_CHARACTER
 /*************************************************
-*          Parse a optional string               *
+*       Decode encoded-character string          *
+*************************************************/
+
+/*
+Encoding definition:
+   hex-pair-seq         = hex-pair *(WSP hex-pair)
+   hex-pair             = 1*2HEXDIG
+
+Arguments:
+  src         points to a hex-pair-seq
+  end         points to its end
+  dst         points to the destination of the decoded octets,
+              optionally to (uschar*)0 for checking only
+
+Returns:      >=0              number of decoded octets
+              -1               syntax error
+*/
+
+static int hex_decode(uschar *src, uschar *end, uschar *dst)
+{
+int decoded=0;
+
+while (src<end)
+  {
+  int l,h;
+
+  if (*src==' ' || *src=='\t') ++src;
+  else if ((src+1)<end && isxdigit(h=tolower(*src)) && isxdigit(l=tolower(*(src+1))))
+    {
+    h=(h>='0' && h<='9') ? h-'0' : 10+(h-'a');
+    l=(l>='0' && l<='9') ? l-'0' : 10+(l-'a');
+    if (dst) *dst++=(h<<4)|l;
+    ++decoded;
+    src+=2;
+    }
+  else return -1;
+  }
+  return decoded;
+}
+
+
+/*************************************************
+*       Decode encoded-character string          *
+*************************************************/
+
+/*
+Encoding definition:
+   unicode-hex-seq      = unicode-hex *(WSP unicode-hex)
+   unicode-hex          = 1*6HEXDIG
+
+   It is an error for a script to use a hexadecimal value that isn't in
+   either the range 0 to D7FF or the range E000 to 10FFFF.
+
+Arguments:
+  src         points to a unicode-hex-seq
+  end         points to its end
+  dst         points to the destination of the decoded octets,
+              optionally to (uschar*)0 for checking only
+
+Returns:      >=0              number of decoded octets
+              -1               syntax error
+              -2               semantic error (character range violation)
+*/
+
+static int unicode_decode(uschar *src, uschar *end, uschar *dst)
+{
+int decoded=0;
+
+while (src<end)
+  {
+  int c,n5,n4,n3,n2,n1,n0;
+
+  if (*src==' ' || *src=='\t') ++src;
+  else if (
+           (src+5)<end
+           && isxdigit(n5=tolower(*src))
+           && isxdigit(n4=tolower(*(src+1)))
+           && isxdigit(n3=tolower(*(src+2)))
+           && isxdigit(n2=tolower(*(src+3)))
+           && isxdigit(n1=tolower(*(src+4)))
+           && isxdigit(n0=tolower(*(src+5)))
+          )
+    {
+    n5=(n5>='0' && n5<='9') ? n5-'0' : 10+(n5-'a');
+    n4=(n4>='0' && n4<='9') ? n4-'0' : 10+(n4-'a');
+    n3=(n3>='0' && n3<='9') ? n3-'0' : 10+(n3-'a');
+    n2=(n2>='0' && n2<='9') ? n2-'0' : 10+(n2-'a');
+    n1=(n1>='0' && n1<='9') ? n1-'0' : 10+(n1-'a');
+    n0=(n0>='0' && n0<='9') ? n0-'0' : 10+(n0-'a');
+    c=(n5<<24)|(n4<<16)|(n3<<12)|(n2<<8)|(n1<<4)|n0;
+    if (!((c>=0 && c<=0xd7ff) || (c>=0xe000 && c<=0x10ffff))) return -2;
+    if (c<128)
+      {
+      if (dst) *dst++=c;
+      ++decoded;
+      }
+    else if (c>=0x80 && c<=0x7ff)
+      {
+        if (dst)
+          {
+          *dst++=192+(c>>6);
+          *dst++=128+(c&0x3f);
+          }
+        decoded+=2;
+      }
+    else if (c>=0x800 && c<=0xffff)
+      {
+        if (dst)
+          {
+          *dst++=224+(c>>12);
+          *dst++=128+((c>>6)&0x3f);
+          *dst++=128+(c&0x3f);
+          }
+        decoded+=3;
+      }
+    else if (c>=0x10000 && c<=0x1fffff)
+      {
+        if (dst)
+          {
+          *dst++=240+(c>>18);
+          *dst++=128+((c>>10)&0x3f);
+          *dst++=128+((c>>6)&0x3f);
+          *dst++=128+(c&0x3f);
+          }
+        decoded+=4;
+      }
+    src+=6;
+    }
+  else return -1;
+  }
+  return decoded;
+}
+
+
+/*************************************************
+*       Decode encoded-character string          *
+*************************************************/
+
+/*
+Encoding definition:
+   encoded-arb-octets   = "${hex:" hex-pair-seq "}"
+   encoded-unicode-char = "${unicode:" unicode-hex-seq "}"
+
+Arguments:
+  encoded     points to an encoded string, returns decoded string
+  filter      points to the Sieve filter including its state
+
+Returns:      1                success
+              -1               syntax error
+*/
+
+static int string_decode(struct Sieve *filter, struct String *data)
+{
+uschar *src,*dst,*end;
+
+src=data->character;
+dst=src;
+end=data->character+data->length;
+while (src<end)
+  {
+  uschar *brace;
+
+  if (
+      Ustrncmp(src,CUS "${hex:",6)==0
+      && (brace=Ustrchr(src+6,'}'))!=(uschar*)0
+      && (hex_decode(src+6,brace,(uschar*)0))>=0
+     )
+    {
+    dst+=hex_decode(src+6,brace,dst);
+    src=brace+1;
+    }
+  else if (
+           Ustrncmp(src,CUS "${unicode:",10)==0
+           && (brace=Ustrchr(src+10,'}'))!=(uschar*)0
+          )
+    {
+    switch (unicode_decode(src+10,brace,(uschar*)0))
+      {
+      case -2:
+        {
+        filter->errmsg=CUS "unicode character out of range";
+        return -1;
+        }
+      case -1:
+        {
+        *dst++=*src++;
+        break;
+        }
+      default:
+        {
+        dst+=unicode_decode(src+10,brace,dst);
+        src=brace+1;
+        }
+      }
+    }
+  else *dst++=*src++;
+  }
+  data->length=dst-data->character;
+  *dst='\0';
+return 1;
+}
+#endif
+
+
+/*************************************************
+*          Parse an optional string              *
 *************************************************/
 
 /*
@@ -1155,6 +1449,11 @@ if (*filter->pc=='"') /* quoted string */
       ++filter->pc;
       /* that way, there will be at least one character allocated */
       data->character=string_cat(data->character,&dataCapacity,&foo,CUS "",1);
+#ifdef ENCODED_CHARACTER
+      if (filter->require_encoded_character
+          && string_decode(filter,data)==-1)
+        return -1;
+#endif
       return 1;
       }
     else if (*filter->pc=='\\' && *(filter->pc+1)) /* quoted character */
@@ -1239,6 +1538,11 @@ else if (Ustrncmp(filter->pc,CUS "text:",5)==0) /* multiline string */
         filter->pc+=2;
 #endif
         ++filter->line;
+#ifdef ENCODED_CHARACTER
+        if (filter->require_encoded_character
+            && string_decode(filter,data)==-1)
+          return -1;
+#endif
         return 1;
         }
       else if (*filter->pc=='.' && *(filter->pc+1)=='.') /* remove dot stuffing */
@@ -2173,6 +2477,48 @@ else if (parse_identifier(filter,CUS "envelope"))
     }
   return 1;
   }
+#ifdef ENOTIFY
+else if (parse_identifier(filter,CUS "valid_notif_method"))
+  {
+  /*
+  valid_notif_method = "valid_notif_method"
+                       <notification-uris: string-list>
+  */
+
+  struct String *uris,*u;
+  int m;
+
+  if (!filter->require_enotify)
+    {
+    filter->errmsg=CUS "missing previous require \"enotify\";";
+    return -1;
+    }
+  if (parse_white(filter)==-1) return -1;
+  if ((m=parse_stringlist(filter,&uris))!=1)
+    {
+    if (m==0) filter->errmsg=CUS "URI string list expected";
+    return -1;
+    }
+  if (exec)
+    {
+    *cond=1;
+    for (u=uris; u->length!=-1 && *cond; ++u)
+      {
+        string_item *recipient;
+        struct String header,body;
+
+        recipient=NULL;
+        header.length=-1;
+        header.character=(uschar*)0;
+        body.length=-1;
+        body.character=(uschar*)0;
+        if (parse_mailto_uri(filter,u->character,&recipient,&header,&body)!=1)
+          *cond=0;
+      }
+    }
+  return 1;
+  }
+#endif
 else return 0;
 }
 
@@ -2479,37 +2825,44 @@ while (*filter->pc)
       }
     if (parse_semicolon(filter)==-1) return -1;
     }
-#ifdef NOTIFY
+#ifdef ENOTIFY
   else if (parse_identifier(filter,CUS "notify"))
     {
     /*
-    notify-command =  "notify" { notify-options } ";"
-    notify-options =  [":method" string]
-                      [":priority" string]
+    notify-command =  "notify" { notify-options } <method: string> ";"
+    notify-options =  [":from" string]
+                      [":importance" <"1" / "2" / "3">]
+                      [":options" 1*(string-list / number)]
                       [":message" string]
     */
 
     int m;
-    struct String method;
-    struct String priority;
+    struct String from;
+    struct String importance;
+    struct String *options;
     struct String message;
+    struct String method;
     struct Notification *already;
     string_item *recipient;
+    struct String header;
     struct String body;
     uschar *envelope_from,*envelope_to;
 
-    if (!filter->require_notify)
+    if (!filter->require_enotify)
       {
-      filter->errmsg=CUS "missing previous require \"notify\";";
+      filter->errmsg=CUS "missing previous require \"enotify\";";
       return -1;
       }
-    method.character=(uschar*)0;
-    method.length=-1;
-    priority.character=(uschar*)0;
-    priority.length=-1;
+    from.character=(uschar*)0;
+    from.length=-1;
+    importance.character=(uschar*)0;
+    importance.length=-1;
+    options=(struct String*)0;
     message.character=(uschar*)0;
     message.length=-1;
     recipient=NULL;
+    header.length=-1;
+    header.character=(uschar*)0;
     body.length=-1;
     body.character=(uschar*)0;
     envelope_from=expand_string("$sender_address");
@@ -2517,27 +2870,32 @@ while (*filter->pc)
     for (;;)
       {
       if (parse_white(filter)==-1) return -1;
-      if (parse_identifier(filter,CUS ":method")==1)
+      if (parse_identifier(filter,CUS ":from")==1)
         {
         if (parse_white(filter)==-1) return -1;
-        if ((m=parse_string(filter,&method))!=1)
+        if ((m=parse_string(filter,&from))!=1)
           {
-          if (m==0) filter->errmsg=CUS "method string expected";
+          if (m==0) filter->errmsg=CUS "from string expected";
           return -1;
           }
-        if (method.length>7 && strncmp(method.character,"mailto:",7)==0)
-          {
-          if (parse_mailto_uri(filter,method.character+7,&recipient,&body)==-1) return -1;
-          }
         }
-      else if (parse_identifier(filter,CUS ":priority")==1)
+      else if (parse_identifier(filter,CUS ":importance")==1)
         {
         if (parse_white(filter)==-1) return -1;
-        if ((m=parse_string(filter,&priority))!=1)
+        if ((m=parse_string(filter,&importance))!=1)
           {
-          if (m==0) filter->errmsg=CUS "priority string expected";
+          if (m==0) filter->errmsg=CUS "importance string expected";
           return -1;
           }
+        if (importance.length!=1 || importance.character[0]<'1' || importance.character[0]>'3')
+          {
+          filter->errmsg=CUS "invalid importance";
+          return -1;
+          }
+        }
+      else if (parse_identifier(filter,CUS ":options")==1)
+        {
+        if (parse_white(filter)==-1) return -1;
         }
       else if (parse_identifier(filter,CUS ":message")==1)
         {
@@ -2550,73 +2908,81 @@ while (*filter->pc)
         }
       else break;
       }
+    if (parse_white(filter)==-1) return -1;
+    if ((m=parse_string(filter,&method))!=1)
+      {
+      if (m==0) filter->errmsg=CUS "missing method string";
+      return -1;
+      }
     if (parse_semicolon(filter)==-1) return -1;
 
-    if (method.length==-1)
+    for (already=filter->notified; already; already=already->next)
       {
-        if ((filter_test != FTEST_NONE && debug_selector != 0) || (debug_selector & D_filter) != 0)
+      if (already->method.length==method.length
+          && (method.length==-1 || strcmp(already->method.character,method.character)==0)
+          && already->importance.length==importance.length
+          && (importance.length==-1 || strcmp(already->importance.character,importance.character)==0)
+          && already->message.length==message.length
+          && (message.length==-1 || strcmp(already->message.character,message.character)==0))
+        break;
+      }
+    if (already==(struct Notification*)0)
+      /* New notification, process it */
+      {
+      if (parse_mailto_uri(filter,method.character,&recipient,&header,&body)!=1)
+        return -1;
+      struct Notification *sent;
+      sent=store_get(sizeof(struct Notification));
+      sent->method=method;
+      sent->importance=importance;
+      sent->message=message;
+      sent->next=filter->notified;
+      filter->notified=sent;
+      if ((filter_test != FTEST_NONE && debug_selector != 0) || (debug_selector & D_filter) != 0)
+        {
+        debug_printf("Notification to `%s'.\n",method.character);
+        }
+#ifndef COMPILE_SYNTAX_CHECKER
+      if (exec && filter_test == FTEST_NONE)
+        {
+        string_item *p;
+        header_line *h;
+        int pid,fd;
+
+        if ((pid = child_open_exim2(&fd,envelope_to,envelope_to))>=1)
           {
-          debug_printf("Ignoring method-less notification.\n");
+          FILE *f;
+          uschar *buffer;
+          int buffer_capacity;
+
+          f = fdopen(fd, "wb");
+          for (h = header_list; h != NULL; h = h->next)
+            if (h->type == htype_received) fprintf(f,"%s",h->text);
+          fprintf(f,"From: %s\n",from.length==-1 ? envelope_to : from.character);
+          for (p=recipient; p; p=p->next) fprintf(f,"To: %s\n",p->text);
+          if (header.length>0) fprintf(f,"%s",header.character);
+          if (message.length==-1)
+            {
+            message.character=US"Notification";
+            message.length=Ustrlen(message.character);
+            }
+          /* Allocation is larger than neccessary, but enough even for split MIME words */
+          buffer_capacity=32+4*message.length;
+          buffer=store_get(buffer_capacity);
+          fprintf(f,"Subject: %s\n\n",parse_quote_2047(message.character, message.length, US"utf-8", buffer, buffer_capacity, TRUE));
+          if (body.length>0) fprintf(f,"%s\n",body.character);
+          fflush(f);
+          (void)fclose(f);
+          (void)child_close(pid, 0);
           }
+        }
+#endif
       }
     else
       {
-      for (already=filter->notified; already; already=already->next)
+      if ((filter_test != FTEST_NONE && debug_selector != 0) || (debug_selector & D_filter) != 0)
         {
-        if (already->method.length==method.length
-            && (method.length==-1 || strcmp(already->method.character,method.character)==0)
-            && already->priority.length==priority.length
-            && (priority.length==-1 || strcmp(already->priority.character,priority.character)==0)
-            && already->message.length==message.length
-            && (message.length==-1 || strcmp(already->message.character,message.character)==0))
-          break;
-        }
-      if (already==(struct Notification*)0)
-        /* New notification, process it */
-        {
-        struct Notification *sent;
-        sent=store_get(sizeof(struct Notification));
-        sent->method=method;
-        sent->priority=priority;
-        sent->message=message;
-        sent->next=filter->notified;
-        filter->notified=sent;
-        if ((filter_test != FTEST_NONE && debug_selector != 0) || (debug_selector & D_filter) != 0)
-          {
-          debug_printf("Notification to `%s'.\n",method.character);
-          }
-#ifndef COMPILE_SYNTAX_CHECKER
-        if (exec)
-          {
-          string_item *p;
-          header_line *h;
-          int pid,fd;
-
-          if ((pid = child_open_exim2(&fd,envelope_to,envelope_to))>=1)
-            {
-            FILE *f;
-
-            f = fdopen(fd, "wb");
-            fprintf(f,"From: %s\n",envelope_to);
-            for (p=recipient; p; p=p->next) fprintf(f,"To: %s\n",p->text);
-            for (h = header_list; h != NULL; h = h->next)
-              if (h->type == htype_received) fprintf(f,"%s",h->text);
-            fprintf(f,"Subject: %s\n",message.length==-1 ? CUS "notification" : message.character);
-            fprintf(f,"\n");
-            if (body.length>0) fprintf(f,"%s\n",body.character);
-            fflush(f);
-            (void)fclose(f);
-            (void)child_close(pid, 0);
-            }
-          }
-#endif
-        }
-      else
-        {
-        if ((filter_test != FTEST_NONE && debug_selector != 0) || (debug_selector & D_filter) != 0)
-          {
-          debug_printf("Repeated notification to `%s' ignored.\n",method.character);
-          }
+        debug_printf("Repeated notification to `%s' ignored.\n",method.character);
         }
       }
     }
@@ -2689,31 +3055,14 @@ while (*filter->pc)
         }
       else if (parse_identifier(filter,CUS ":from")==1)
         {
-        int start, end, domain;
-        uschar *error,*ss;
-
         if (parse_white(filter)==-1) return -1;
         if ((m=parse_string(filter,&from))!=1)
           {
           if (m==0) filter->errmsg=CUS "from string expected";
           return -1;
           }
-        if (from.length>0)
-          {
-          ss = parse_extract_address(from.character, &error, &start, &end, &domain,
-            FALSE);
-          if (ss == NULL)
-            {
-            filter->errmsg=string_sprintf("malformed address \"%s\" in "
-              "Sieve filter: %s", from.character, error);
-            return -1;
-            }
-          }
-        else
-          {
-          filter->errmsg=CUS "empty :from address in Sieve filter";
+        if (check_mail_address(filter,&from)!=1)
           return -1;
-          }
         }
       else if (parse_identifier(filter,CUS ":addresses")==1)
         {
@@ -2935,11 +3284,14 @@ filter->line=1;
 filter->keep=1;
 filter->require_envelope=0;
 filter->require_fileinto=0;
+#ifdef ENCODED_CHARACTER
+filter->require_encoded_character=0;
+#endif
 #ifdef ENVELOPE_AUTH
 filter->require_envelope_auth=0;
 #endif
-#ifdef NOTIFY
-filter->require_notify=0;
+#ifdef ENOTIFY
+filter->require_enotify=0;
 filter->notified=(struct Notification*)0;
 #endif
 #ifdef SUBADDRESS
@@ -3007,11 +3359,14 @@ while (parse_identifier(filter,CUS "require"))
     {
     if (eq_octet(check,&str_envelope,0)) filter->require_envelope=1;
     else if (eq_octet(check,&str_fileinto,0)) filter->require_fileinto=1;
+#ifdef ENCODED_CHARACTER
+    else if (eq_octet(check,&str_encoded_character,0)) filter->require_encoded_character=1;
+#endif
 #ifdef ENVELOPE_AUTH
     else if (eq_octet(check,&str_envelope_auth,0)) filter->require_envelope_auth=1;
 #endif
-#ifdef NOTIFY
-    else if (eq_octet(check,&str_notify,0)) filter->require_notify=1;
+#ifdef ENOTIFY
+    else if (eq_octet(check,&str_enotify,0)) filter->require_enotify=1;
 #endif
 #ifdef SUBADDRESS
     else if (eq_octet(check,&str_subaddress,0)) filter->require_subaddress=1;
